"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.APIReferenceModel = void 0;
const semantics_entities_semantic_schema_1 = require("@teambit/semantics.entities.semantic-schema");
const component_id_1 = require("@teambit/component-id");
class APIReferenceModel {
    internalAPIKey(schema) {
        return this.generateInternalAPIKey(schema.location.filePath, schema.name);
    }
    generateInternalAPIKey(filePath, name = '') {
        return `${filePath}/${name}`;
    }
    constructor(_api, _renderers) {
        this._api = _api;
        this.taggedAPINodes = [];
        this.componentId = _api.componentId;
        this.apiNodes = this.mapToAPINode(_api, _renderers, this.componentId);
        this.apiByType = this.groupByType(this.apiNodes);
        this.apiByName = this.groupByName(this.apiNodes);
        this.taggedAPINodes = this.mapTaggedAPINode(_api, _renderers, this.componentId);
    }
    mapTaggedAPINode(api, renderers, componentId) {
        const { taggedModuleExports } = api;
        const defaultRenderers = renderers.filter((renderer) => renderer.default);
        const nonDefaultRenderers = renderers.filter((renderer) => !renderer.default);
        return taggedModuleExports
            .map((schemaNode) => ({
            componentId,
            api: schemaNode,
            exported: true,
            renderer: nonDefaultRenderers.find((renderer) => renderer.predicate(schemaNode)) ||
                defaultRenderers.find((renderer) => renderer.predicate(schemaNode)),
        }))
            .filter((schemaNode) => schemaNode.renderer);
    }
    mapToAPINode(api, renderers, componentId) {
        const { internals } = api;
        const { exports: exportedSchemaNodes } = api.module;
        const exportedInternalKeySet = new Set(exportedSchemaNodes.map((schemaNode) => this.internalAPIKey(schemaNode)));
        const internalSchemaNodes = internals
            .reduce((acc, next) => {
            return acc.concat([...next.internals]);
        }, new Array())
            .filter((schemaNode) => !exportedInternalKeySet.has(this.internalAPIKey(schemaNode)));
        const defaultRenderers = renderers.filter((renderer) => renderer.default);
        const nonDefaultRenderers = renderers.filter((renderer) => !renderer.default);
        return exportedSchemaNodes
            .map((schemaNode) => ({
            componentId,
            api: semantics_entities_semantic_schema_1.ExportSchema.isExportSchema(schemaNode) ? schemaNode.exportNode : schemaNode,
            alias: semantics_entities_semantic_schema_1.ExportSchema.isExportSchema(schemaNode) ? schemaNode.alias : undefined,
            exported: true,
            renderer: nonDefaultRenderers.find((renderer) => renderer.predicate(semantics_entities_semantic_schema_1.ExportSchema.isExportSchema(schemaNode) ? schemaNode.exportNode : schemaNode)) ||
                defaultRenderers.find((renderer) => renderer.predicate(semantics_entities_semantic_schema_1.ExportSchema.isExportSchema(schemaNode) ? schemaNode.exportNode : schemaNode)),
        }))
            .concat(internalSchemaNodes.map((schemaNode) => ({
            componentId,
            api: schemaNode,
            exported: false,
            renderer: nonDefaultRenderers.find((renderer) => renderer.predicate(schemaNode)) ||
                defaultRenderers.find((renderer) => renderer.predicate(schemaNode)),
        })))
            .filter((schemaNode) => schemaNode.renderer);
    }
    getByType(type) {
        var _a;
        return (_a = this.apiByType.get(type)) !== null && _a !== void 0 ? _a : [];
    }
    getByName(node) {
        var _a;
        if (!node.name)
            return undefined;
        return (_a = this.apiByName.get(node.name)) !== null && _a !== void 0 ? _a : this.apiByName.get(this.internalAPIKey(node));
    }
    groupByType(apiNodes) {
        return apiNodes.reduce((accum, next) => {
            const existing = accum.get(next.renderer.nodeType) || [];
            accum.set(next.renderer.nodeType, existing.concat(next));
            return accum;
        }, new Map());
    }
    groupByName(apiNodes) {
        return apiNodes.reduce((accum, next) => {
            const name = next.alias || next.api.name;
            if (!name)
                return accum;
            const key = next.exported ? name : this.internalAPIKey(next.api);
            accum.set(key, next);
            return accum;
        }, new Map());
    }
    static from(result, renderers = []) {
        try {
            const apiSchema = semantics_entities_semantic_schema_1.APISchema.fromObject(result.getHost.getSchema);
            return new APIReferenceModel(apiSchema, renderers);
        }
        catch (e) {
            return new APIReferenceModel(semantics_entities_semantic_schema_1.APISchema.empty(component_id_1.ComponentID.fromObject(result.getHost.getSchema.componentId)), renderers);
        }
    }
}
exports.APIReferenceModel = APIReferenceModel;
//# sourceMappingURL=api-reference-model.js.map